/*
  ticktalk
  
  A clock that displays time in various languages
  
  History -
  
  08/05/92: Paul E. Black <beehive!paul@cirrus.com>
  Added Klingon (v3.1)

  04/01/92: Art Mellor
  Bumped version to 3.0 and posted to alt.sources
  04/01/92: Art Mellor
  Added Baby Talk (v2.6)
  Ignores approximate mode and uses -before flag to control whether
  the big hand or little hand gets precedence.

  03/11/92: Massimo Dal Zotto <dz@irst.it>
  Added Italian (v2.5)

  03/09/92: kbriggs@mundoe.maths.mu.OZ.AU (Keith Briggs rba8 7088)
  Minor bug in german mode.
  03/09/92: Kiem (V.K.) Ngo <KINGO@BNR.CA>
  Added VietNamese (v2.4)
  03/09/92: Art Mellor
  Fixed a minor bug in PigLatin.

  03/05/92: Ole Arntzen (olea@ii.uib.no)
  Norwegian version (v2.3)
  03/05/92: H}kan Lennest}l <hakanl@lulea.trab.se>
  Swedish version (v2.2) with -latin1 flag
  03/05/92: Art Mellor
  Changed -language kanji to be extra -kanji flag for -language japanese

  03/04/92: Matsushita Makoto <matusita@exp.ics.osaka-u.ac.jp>
  Bug fixes to Kanji version.
  03/04/92: Xavier Leroy <xleroy@pomerol.inria.fr>
  More bug fixes to that pesky french version (v2.1)

  03/03/92: Art Mellor
  Bumped version to 2.0 and posted to alt.sources

  03/03/92: Johan van der Werf <jvdwerf@mmra1.ms.philips.nl>
  Added Japenese and Kanji versions (v1.7)
  03/03/92: Jochen Linz <jlinz@krokus.han.de>
  Added German version (v1.5)

  02/20/92: Chris Hadley <ckh@cl.cam.ac.uk>
  Added spanish version (v1.4)

  02/19/92: John 'Pig' Reardon <badger@inmet.camb.inmet.com>
  Added Pig Latin (v1.3) per suggestion of Hollie Schmidt
  <winter!hschmidt@eddie.mit.edu>

  02/18/92: Richard van de Stadt <stadt@cs.utwente.nl>
  Latest dutch version that I'm going with (unless everyone complains)
  Adds -half option. I appreciate the other dutch versions, but this one
  seems pretty good and Richard was first.

  02/17/92: Richard van de Stadt <stadt@cs.utwente.nl>
  			Johan Thole <thole@CS.RULIMBURG.NL>
			egroenen@nl.oracle.com (Edward Groenenberg)
			Stefan Linnemann <Stefan.Linnemann@rulcvx.LeidenUniv.nl>
  Various dutch version merging (fun to do in a language that I don't speak)
  Added them as dutch[1-3] and sent to stadt for feedback.
  02/17/92: Vesa K|pp{ <k103063@cc.tut.fi>
  Added Finnish version v1.2
  02/17/92: Bernard Moret <moret@tiguex.cs.unm.edu>
  Changes to french mode to fix errors.

  02/05/92: Art Mellor
  Fixed testtalk to not end 1 minute early
  Changed languages to use -language flag as suggested
    by paul@delfin.com (Paul Black)
  Added more meaningful error messages.
  02/05/92: Richard van de Stadt <stadt@cs.utwente.nl>
  Added Dutch version v1.1

  01/20/92: Art Mellor
  Released Version 1.0
  01/20/92: Kaveh Kardan
  Added before mode for french and fixed some typos

  01/07/91: Kaveh Kardan
  Added approximate mode to french and made some corrections.

  01/02/91: Alan Steele (neptune@fenchurch.mit.edu)
  Added code to imbed icon in application

  12/31/91: Art Mellor
  Added -noampm flag at request of Dave Baker (dvb@inmet.inmet.com)
  12/31/91: Linda Matthew (linder@watson.ibm.com), Art Mellor
  Added Afrikaans (Linda language, Art coding) v0.4

  12/27/91:	Art Mellor
  Added TEXT,TEST mode
  Added -update,-before,-approximate,-help flags

  12/26/91:	Art Mellor
  Changed French to runtime option

  11/29/91:	Kaveh Kardan (kaveh%taarna%icule@larry.mcrcim.mcgill.edu)
  Added French version as compile time option

  5/6/91: 	Art Mellor (art@cayman.com)
  Initial version
*/

/*
  There are 3 types of compiles:

  For X windows - has no defines of TEST or TEXT
  For testing - has TEST and TEXT defined
  For text output - has TEXT defined
*/

#ifdef TEST
#	define TEXT
#endif TEST

#include <stdio.h>
#include <time.h>
#include <ctype.h>

#ifndef TEXT
#	include <X11/Intrinsic.h>
#	include <X11/StringDefs.h>
#	include <X11/Shell.h>
#	include <X11/Xaw/Label.h>
#	include "xticktalk.xbm"
#else
#	define FALSE			0
#	define TRUE				(!FALSE)
#endif !TEXT

#define MAX_TIMESTRING		256		/* size of strings */
#define MAXARGS				10		/* for X widgets */

/* These are all the languages defined */
#define ENGLISH		0
#define FRENCH		1
#define AFRIKAANS	2
#define DUTCH		3
#define FINNISH		4
#define PIGLATIN	5
#define SPANISH		6
#define GERMAN		7
#define JAPANESE	8
#define SWEDISH		9
#define NORWEGIAN	10
#define VIETNAMESE	11
#define ITALIAN		12
#define BABYTALK	13
#define KLINGON		14

int latin1 = FALSE;					/* Used with -language swedish/norwegian */
int kanji = FALSE;					/* Used with -language japanese */
int half = FALSE;					/* alternate time mode for dutch */
int timeOut = 30;					/* how often to update X clock display */
int language = -1;					/* language chosen by user */
int before = FALSE;					/* use "before" syntax when min > 30 */
int approximate = FALSE;			/* round to nearest 5 minutes */
int noampm = FALSE;					/* don't put am/pm on the end */
#ifdef TEXT
int hour = -1, minute = -1;			/* optional args for TEXT/TEST modes */
#endif TEXT

char *programName;					/* store argv[0] here for error messages */
#define VERSION_MAJOR 3				/* version numbers */
#define VERSION_MINOR 1
#define HELPADDR	  "art@cayman.com"
char versionString[MAX_TIMESTRING];

/*
  Call this routine to help unclued users
*/
void Usage()
{
  static char *usageStrings[] = {
	" [-help] [-version] [-before] [-language <language>] \\\n",
	"        [-approximate] [-noampm] [-half (dutch)] [-kanji (japanese)]\\\n",
	"        [-latin1 (swedish/norwegian)]",
#ifdef TEXT
	" [hour(0-23) min(0-59)]\n",
#else
	" [-update secs] [X Options]\n",
#endif TEXT
	"  where <language> is one of the following:\n",
	"    english | french | afrikaans | dutch | finnish |\n",
	"    piglatin | spanish | german | japanese | swedish\n",
	"    norwegian | vietnamese | italian | babytalk | klingon\n",
	(char *)0};
  char **uPtr = usageStrings;
  
  fprintf(stderr,"usage: %s ", programName);
  while(*uPtr){
	fprintf(stderr, "%s", *uPtr);
	uPtr++;
  }
}

/*
  Call this routine to explain things in more detail
*/
void PrintHelp()
{
  static char *helpStrings[] = {
	"Options:\n",
	"-help           Print this text\n",
	"-version        Print the current version and exit\n",
	"-before         Print time after the half hour as relative to next hour\n",
	"-approximate    Round time off to nearest 5 minutes\n",
	"-language <language>   The language to display the time in\n",
	"-noampm         Don't put am/pm on the time string (if present otherwise)\n",
	"-half           Alternate time mode for dutch\n",
	"-kanji          Uses kanji character set with japanese (e.g. with kterm)\n",
	"-latin1         Uses the Latin-1 character set with swedish/norwegian\n",
#ifdef TEXT
	"hour min        Print the time corresponding to hour:min\n",
#ifdef TEST
	"With no arguments all time values are printed to stdout for verification\n",
#else
	"With no arguments prints the current time to stdout\n",
#endif TEST
#else
	"-update secs    How frequently to update the display\n",
	"X Options       Standard X windows options\n",
#endif TEXT
	(char *)0};
  char **hPtr = helpStrings;
  
  Usage();
  while(*hPtr){
	fprintf(stderr, "  %s", *hPtr);
	hPtr++;
  }
}

/*
  Handle the args to the function
*/
void ProcessArgs(argc, argv)
int argc;
char *argv[];
{
  programName = argv[0];
  sprintf(versionString,"%s v%d.%d",programName,VERSION_MAJOR,VERSION_MINOR);
  
  argv++;
  while(--argc){
	if(**argv != '-'){
	  break;
	}
	if(strcmp("-language", *argv) == 0){
	  --argc; argv++;
	  if(!argc || (language != -1)){
		if(!argc){
		  fprintf(stderr, "%s: -language expects a language\n", programName);
		}
		else{
		  fprintf(stderr, "%s: only one language can be specified at a time\n",
				  programName);
		}
		Usage();
		exit(-1);
	  }
	  if(strcmp("english", *argv) == 0){
		language = ENGLISH;
	  }
	  else if(strcmp("french", *argv) == 0){
		language = FRENCH;
	  }
	  else if(strcmp("afrikaans", *argv) == 0){
		language = AFRIKAANS;
	  }
	  else if(strcmp("dutch", *argv) == 0){
		language = DUTCH;
	  }
	  else if(strcmp("finnish", *argv) == 0){
		language = FINNISH;
	  }
	  else if(strcmp("piglatin", *argv) == 0){
		language = PIGLATIN;
	  }
	  else if(strcmp("spanish", *argv) == 0){
		language = SPANISH;
	  }
	  else if(strcmp("german", *argv) == 0){
		language = GERMAN;
	  }
	  else if(strcmp("japanese", *argv) == 0){
		language = JAPANESE;
	  }
	  else if(strcmp("swedish", *argv) == 0){
		language = SWEDISH;
	  }
	  else if(strcmp("norwegian", *argv) == 0){
		language = NORWEGIAN;
	  }
	  else if(strcmp("vietnamese", *argv) == 0){
		language = VIETNAMESE;
	  }
	  else if(strcmp("italian", *argv) == 0){
		language = ITALIAN;
	  }
	  else if(strcmp("babytalk", *argv) == 0){
		language = BABYTALK;
	  }
	  else if(strcmp("klingon", *argv) == 0){
		language = KLINGON;
	  }
	  else{
		fprintf(stderr, "%s: unknown language '%s'\n", programName, *argv);
		Usage();
		exit(-1);
	  }
	}
	else if(strcmp("-half", *argv) == 0){
	  half = TRUE;
	}
	else if(strcmp("-kanji", *argv) == 0){
	  kanji = TRUE;
	}
	else if(strcmp("-latin1", *argv) == 0){
	  latin1 = TRUE;
	}
	else if(strcmp("-version", *argv) == 0){
	  printf("%s (bugs/enhancements to %s)\n", versionString, HELPADDR);
	  exit(0);
	}
	else if(strcmp("-before", *argv) == 0){
	  before = TRUE;
	}
	else if(strcmp("-help", *argv) == 0){
	  PrintHelp();
	  exit(0);
	}
	else if(strcmp("-approximate", *argv) == 0){
	  approximate = TRUE;
	}
	else if(strcmp("-noampm", *argv) == 0){
	  noampm = TRUE;
	}
#ifndef TEXT
	else if(strcmp("-update", *argv) == 0){
	  --argc; argv++;
	  if(!argc || !IsNumber(*argv) || ((timeOut = atoi(*argv)) <= 0)){
		fprintf(stderr, "%s: update must be a number greater than 0\n",
				programName);
		Usage();
		exit(-1);
	  }
	}
#endif !TEXT
	else{
	  fprintf(stderr, "%s: unknown option '%s'\n", programName, *argv);
	  Usage();
	  exit(-1);
	}
	argv++;
  }
  
  /* Default to English */
  if(language == -1){
	language = ENGLISH;
  }

  /* Check for language specific flag misuse */
  if(half && (language != DUTCH)){
	fprintf(stderr, "%s: -half option only valid with -language dutch\n",
			programName);
	Usage();
	exit(-1);
  }

  if(kanji && (language != JAPANESE)){
	fprintf(stderr, "%s: -kanji option only valid with -language japanese\n",
			programName);
	Usage();
	exit(-1);
  }

  if(latin1 && ((language != SWEDISH) && (language != NORWEGIAN))){
	fprintf(stderr,
		 "%s: -latin1 option only valid with -language swedish or norwegian\n",
		    programName);
	Usage();
	exit(-1);
  }

  if(argc == 0){
	return;
  }
#ifdef TEXT
  else if(argc == 2){
	if(IsNumber(argv[0]) && IsNumber(argv[1])){
	  hour = atoi(argv[0]);
	  minute = atoi(argv[1]);
	  if((hour < 24) && (minute < 60)){
		return;
	  }
	  else{
		fprintf(stderr, "%s: time arguments out of range [0-23] [0-59]\n",
				programName);
	  }
	}
	else{
	  fprintf(stderr, "%s: time arguments must be numbers\n", programName);
	}
	Usage();
	exit(-1);
  }
#endif TEXT
  
  fprintf(stderr, "%s: wrong number of arguments\n", programName);
  Usage();
  exit(-1);
}

#ifndef TEXT
/*
  Stuff timeString into the widget label
*/
SetLabelString(label, timeString)
Widget label;
char *timeString;
{
  Arg args[MAXARGS];
  int nargs;
  
  nargs = 0;
  XtSetArg(args[nargs],XtNlabel,timeString); nargs++;
  XtSetValues(label, args, nargs);
}

/*
  Callback to set the time string every so often
*/
XtTimerCallbackProc CheckTime(label, ignored)
Widget label;
XtIntervalId *ignored;
{
  char timeString[MAX_TIMESTRING];
  
  GetTimeString(language, timeString);
  SetLabelString(label, timeString);
  XtAddTimeOut(timeOut * 1000, CheckTime, (caddr_t)label);
}

void CreateIcon(shell)
Widget shell;
{
  Pixmap icon_pixmap = (Pixmap) 0;
  Arg arg;
  
  /* User sets iconPixmap resource, converter does the right thing.. */
  XtSetArg(arg, XtNiconPixmap, &icon_pixmap);
  XtGetValues(shell, &arg, 1);
  if (icon_pixmap == (Pixmap) 0) {
	XtSetArg(arg, XtNiconPixmap,
			 XCreateBitmapFromData(
			  XtDisplay(shell), XtScreen(shell)->root,
			  xticktalk_bits, xticktalk_width, xticktalk_height));
	XtSetValues(shell, &arg, 1);
  }
}

main(argc, argv)
int argc;
char *argv[];
{
  Widget shell, label;
  Arg args[MAXARGS];
  int nargs, i;
  
  /* Create shell and process X options */
  shell = XtInitialize(argv[0], versionString,
					   NULL, 0, &argc, argv);
  nargs = 0;
  XtSetArg(args[nargs],XtNallowShellResize,True); nargs++;
  XtSetValues(shell,args,nargs);
  
  /* Process remaining args */
  ProcessArgs(argc, argv);
  
  /* Create icon */
  CreateIcon(shell);

  /* Make the label */
  nargs = 0;
  label = XtCreateManagedWidget("label", labelWidgetClass, shell,
								args, nargs);
  
  /* Set the timer going */
  CheckTime((caddr_t)label, (XtIntervalId *)NULL);
  
  /* Put 'em up */
  XtRealizeWidget(shell);
  XtMainLoop();
}
#else

main(argc, argv)
int argc;
char *argv[];
{
  int h, m, len, longest = 0;
  char timeString[MAX_TIMESTRING], longTime[MAX_TIMESTRING];
  
  /* Process args */
  ProcessArgs(argc, argv);
  
  if(hour != -1){
	Time2Word(language, hour, minute, timeString);
	printf("%s\n", timeString);
	exit(0);
  }
#ifdef TEST
  for(h = 0, m = 0; !((h == 24) && (m == 0));){
	Time2Word(language, h, m, timeString);
	printf("%s\n", timeString);
	if((len = strlen(timeString)) > longest){
	  strcpy(longTime, timeString);
	  longest = len;
	}
	m = (m + 1)%60;
	if(m == 0){
	  h++;
	}
  }
  printf("Longest: %s @ %d characters\n", longTime, longest);
#else
  GetTimeString(language, timeString);
  printf("%s\n", timeString);
#endif TEST
  exit(0);
}

#endif !TEXT

/*
  Increment the hour (based on 12 hour clock with am/pm flag)
  AM/PM changes at the 11/12 border
*/
void TimeIncrement(hPtr, pmPtr)
int *hPtr, *pmPtr;
{
  if(*hPtr == 11){
	*pmPtr = !*pmPtr;
  }
  *hPtr = *hPtr%12 + 1;
}

/*
  Is this string all digits?
*/
IsNumber(str)
char *str;
{
  char *ptr = str;
  
  while(*ptr && isdigit(*ptr)) ptr++;
  return(*ptr == '\0');
}

/*
  Stuff the current time in as a string in specified language
*/
GetTimeString(tongue, string)
int tongue;
char *string;
{
  long t;
  struct tm *timeVal;
  
  t = time(0);
  timeVal = localtime(&t);
  Time2Word(tongue, timeVal->tm_hour, timeVal->tm_min, string);
}

/*
  Convert the specified time into a string in the specified language
*/
Time2Word(tongue, hour, minutes, word)
int tongue, hour, minutes;
char *word;
{
  switch(tongue){
  case ENGLISH:
	return Time2WordEnglish(hour, minutes, word);
  case FRENCH:
	return Time2WordFrench(hour, minutes, word);
  case AFRIKAANS:
	return Time2WordAfrikaans(hour, minutes, word);
  case DUTCH:
	return Time2WordDutch(hour, minutes, word);
  case FINNISH:
	return Time2WordFinnish(hour, minutes, word);
  case PIGLATIN:
	return Time2WordPigLatin(hour, minutes, word);
  case SPANISH:
	return Time2WordSpanish(hour, minutes, word);
  case GERMAN:
	return Time2WordGerman(hour, minutes, word);
  case JAPANESE:
	if(kanji){
	  return Time2WordKanji(hour, minutes, word);  
	}
	return Time2WordJapanese(hour, minutes, word);
  case SWEDISH:
	return Time2WordSwedish(hour, minutes, word);
  case NORWEGIAN:
	return Time2WordNorwegian(hour, minutes, word);
  case VIETNAMESE:
	return Time2WordVietNamese(hour, minutes, word);
  case ITALIAN:
	return Time2WordItalian(hour, minutes, word);
  case BABYTALK:
	return Time2WordBabyTalk(hour, minutes, word);
  case KLINGON:
	return Time2WordKlingon(hour, minutes, word);
  default:
	fprintf(stderr, "Unknown language: %d\n", tongue);
	Usage();
	exit(-1);
  }
}

/*****************************************************************************
  English Time Routines by Art Mellor <art@cayman.com>
 *****************************************************************************/

/*
  Converts a number from 0 to 59 into an english string equivalent
*/
Number2WordEnglish(number, word)
int number;
char *word;
{
  static char *underTwenty[] = {
	"Zero", "One", "Two", "Three", "Four", "Five", "Six", "Seven",
	"Eight", "Nine", "Ten", "Eleven", "Twelve", "Thirteen", "Fourteen",
	"Fifteen", "Sixteen", "Seventeen", "Eighteen", "Nineteen"
	  };
  static char *overNineteen[] = {
	"", "Teen", "Twenty", "Thirty", "Forty", "Fifty"
	  };
  int guzzinto, remainder;
  
  *word = '\0';

  if ((number < 0) || (number > 59)){
	return FALSE;
  }
  
  if (number < 20){
	strcat(word, underTwenty[number]);
	return TRUE;
  }
  
  guzzinto = number/10;
  remainder = number%10;
  
  strcat(word, overNineteen[guzzinto]);
  
  if(remainder != 0){
	strcat(word, "-");
	strcat(word, underTwenty[remainder]);
  }
  
  return TRUE;
}

/*
  Turns 24 hour time (with minutes) into english representation
*/
Time2WordEnglish(hour, minutes, word)
int hour, minutes;
char *word;
{
  int pm, rounded = 0;
  char hourWord[MAX_TIMESTRING], minuteWord[MAX_TIMESTRING];
  
  *word = '\0';
  
  pm = (hour > 11);
  
  if (pm) {
	hour -= 12;
  }
  if (hour == 0){
	hour = 12;
  }
  
  /* Round to nearest 5 minutes */
  if(approximate){
	if((rounded = minutes%5) >= 3){
	  minutes += (5 - rounded);
	  /* If round to next hour, handle correctly */
	  if(minutes == 60){
		minutes = 0;
		TimeIncrement(&hour, &pm);
	  }
	}
	else if(rounded > 0){
	  minutes -= rounded;
	}
	
	strcat(word, rounded?"About ":"");
  }
  
  Number2WordEnglish(hour, hourWord);
  Number2WordEnglish(minutes, minuteWord);
  
  /*
	Try to do some cute things with the time
	*/
  if(minutes == 0){
	if(hour == 12){
	  strcat(word, pm?"Noon":"Midnight");
	  return TRUE;
	}
	else{
	  strcat(word, hourWord);
	  strcat(word, " O'Clock");
	}
  }
  else if((minutes >= 1) && (minutes <= 9)){
	strcat(word, minuteWord);
	strcat(word, " Past ");
	strcat(word, hourWord);
  }
  else if(minutes == 15){
	strcat(word, "Quarter Past ");
	strcat(word, hourWord);
  }
  else if(minutes == 30){
	strcat(word, "Half Past ");
	strcat(word, hourWord);
  }
  else if(minutes == 45){
	strcat(word, "Quarter To ");
	TimeIncrement(&hour, &pm);
	Number2WordEnglish(hour, hourWord);
	strcat(word, hourWord);
  }
  else if((minutes > 30) && before){
	TimeIncrement(&hour, &pm);
	minutes = 60 - minutes;
	Number2WordEnglish(hour, hourWord);
	Number2WordEnglish(minutes, minuteWord);
	strcat(word, minuteWord);
	strcat(word, " To ");
	strcat(word, hourWord);
  }
  else{
	strcat(word, hourWord);
	strcat(word, " ");
	strcat(word, minuteWord);
  }

  if(!noampm){
	strcat(word, pm?" pm":" am");
  }
  
  return TRUE;
}

/******************************************************************************
  Baby Talk time routines by Art Mellor (art@cayman.com)
******************************************************************************/

/*
  Turns 24 hour time (with minutes) into baby talk representation
*/
Time2WordBabyTalk(hour, minutes, word)
int hour, minutes;
char *word;
{
  int pm, rounded = 0;
  char hourWord[MAX_TIMESTRING], minuteWord[MAX_TIMESTRING];
  char bigHand[MAX_TIMESTRING], littleHand[MAX_TIMESTRING];
  
  *word = '\0';
  *bigHand = *littleHand = *word;
  
  pm = (hour > 11);
  
  if (pm) {
	hour -= 12;
  }
  if (hour == 0){
	hour = 12;
  }
  
  /* No approximate time, it's all approximate */

  /* Round to nearest 5 minutes */
  if((rounded = minutes%5) >= 3){
	minutes += (5 - rounded);
	/* If round to next hour, handle correctly */
	if(minutes == 60){
	  minutes = 0;
	  TimeIncrement(&hour, &pm);
	}
  }
  else if(rounded > 0){
	minutes -= rounded;
  }
  
  /* Turn minutes into 1 - 12 */
  minutes /= 5;
  if(minutes == 0){
	minutes = 12;
  }

  Number2WordEnglish(hour, hourWord);
  Number2WordEnglish(minutes, minuteWord);
  
  strcat(bigHand, "The Big Hand Is On The ");
  strcat(bigHand, minuteWord);

  strcat(littleHand, "The Little Hand Is On The ");
  strcat(littleHand, hourWord);

  strcat(word, before?bigHand:littleHand);
  strcat(word, " And ");
  strcat(word, before?littleHand:bigHand);

  if(!noampm){
	strcat(word, pm?" At Night":" In The Morning");
  }
  
  return TRUE;
}

/*****************************************************************************
			French Time Routines
			by Kaveh Kardan <kaveh%taarna%icule@larry.mcrcim.mcgill.edu>
*****************************************************************************/

Number2WordFrench(number, word)
int number;
char *word;
{
  static char *underTwenty[] = {
	"Zero", "Un", "Deux", "Trois", "Quatre", "Cinq", "Six", "Sept",
	"Huit", "Neuf", "Dix", "Onze", "Douze", "Treize", "Quatorze",
	"Quinze", "Seize", "Dix-Sept", "Dix-Huit", "Dix-Neuf"
	  };
  static char *overNineteen[] = {
	"", "", "Vingt", "Trente", "Quarante", "Cinquante"
	  };
  int guzzinto, remainder;
  
  *word = '\0';
  
  if ((number < 0) || (number > 59)){
	return FALSE;
  }
  
  if (number < 20){
	strcat(word, underTwenty[number]);
	return TRUE;
  }
  
  guzzinto = number/10;
  remainder = number%10;
  
  strcpy(word, overNineteen[guzzinto]);
  
  if(remainder != 0){
	if (remainder == 1)
		{
          strcat(word, "-et-");
		}
	else
		{
          strcat(word, "-");
		}
	strcat(word, underTwenty[remainder]);
  }
  
  return TRUE;
}

/*
 Turns 24 hour time (with minutes) into French representation
*/
Time2WordFrench(hour, minutes, word)
int hour, minutes;
char *word;
{
  int pm, nexthour, hour12, nexthour12, rounded = 0;
  char hourWord[MAX_TIMESTRING], minuteWord[MAX_TIMESTRING];
  
  /* leave hour in 24 hour format; it's the FRENCH way! */
  /* but only for exact time; if approximate is selected, then use 12 hour */
  
  *word = '\0';
  
  if(approximate){
	if((rounded = minutes%5) >= 3){
	  minutes += (5 - rounded);
	  if(minutes == 60){
		minutes = 0;
		hour = (hour+1)%24;
	  }
	}
	else if(rounded > 0){
	  minutes -= rounded;
	}
  }
  
  nexthour = (hour+1)%24;
  
  if (approximate && (hour > 11)) {
    hour12 = hour-12;
    nexthour12 = nexthour-12;
  }
  else {
    hour12 = hour;
    nexthour12 = nexthour;
  }
  
  if (hour == 0){
	strcpy(hourWord, "Minuit ");
  }
  else if (hour == 12){
	strcpy(hourWord, "Midi ");
  }
  else{
	Number2WordFrench(hour12, hourWord);
  }
  Number2WordFrench(minutes, minuteWord);
  
  /*
	Try to do some cute things with the time
	*/
  if (!strcmp(hourWord, "Un")){
	strcpy(hourWord, "Une");
  }
  if (!strcmp(hourWord, "Vingt-et-Un")){
	strcpy(hourWord, "Vingt-et-Une");
  }
  if(minutes == 0){
	if(hour == 12){
	  strcat(word, "Midi");
	}
	else{
	  strcat(word, hourWord);
	  if (!strcmp(hourWord, "Une")){
		strcat(word, " Heure");
	  }
	  else if (strcmp(hourWord, "Midi ") &&
			   strcmp(hourWord, "Minuit ")){
		strcat(word, " Heures");
	  }
	}
  }
  else if(minutes == 15){
	strcat(word, hourWord);
	if (!strcmp(hourWord, "Une")){
	  strcat(word, " Heure Et Quart");
	}
	else if (!strcmp(hourWord, "Midi ") || !strcmp(hourWord, "Minuit ")){
	  strcat(word, "Et Quart");
	}
	else{
	  strcat(word, " Heures Et Quart");
	}
  }
  else if(minutes == 30){
	strcat(word, hourWord);
	if (!strcmp(hourWord, "Une")){
	  strcat(word, " Heure Et Demi");
	}
	else if (!strcmp(hourWord, "Midi ") || !strcmp(hourWord, "Minuit ")){
	  strcat(word, "Et Demi");
	}
	else{
	  strcat(word, " Heures Et Demi");
	}
  }
  else if(minutes == 45){
	hour++;
	hour = hour%24;
	
	if (approximate && (hour > 11)) {
	  hour12 = hour-12;
	}
	else {
	  hour12 = hour;
	}
	
	if (hour == 0){
	  strcat(word, "Minuit Moins le Quart"); /* !!! */
	}
	else if (hour == 1){
	  strcat(word, "Une Heure Moins le Quart"); /* !!! */
	}
	else if (hour == 12){
	  strcat(word, "Midi Moins le Quart"); /* !!! */
	}
	else{
	  Number2WordFrench(hour12, hourWord);
	  strcat(word, hourWord);
	  strcat(word, " Heures Moins le Quart"); /* !!! */
	}
  }
  else if((minutes > 30) && before){
	hour++;
	hour = hour%24;
	minutes = 60 - minutes;
	
	if (approximate && (hour > 11)) {
	  hour12 = hour-12;
	}
	else {
	  hour12 = hour;
	}
	
	if (hour == 0){
	  strcpy(hourWord, "Minuit ");
	}
	else if (hour == 12){
	  strcpy(hourWord, "Midi ");
	}
	else{
	  Number2WordFrench(hour12, hourWord);
	  if (!strcmp(hourWord, "Un")){
		strcpy(hourWord, "Une");
	  }
	  if (!strcmp(hourWord, "Vingt-et-Un")){
		strcpy(hourWord, "Vingt-et-Une");
	  }
	}
	
	strcat(word, hourWord);
	
	if (!strcmp(hourWord, "Une")){
	  strcat(word, " Heure");
	}
	else if (strcmp(hourWord, "Midi ") &&
			 strcmp(hourWord, "Minuit ")){
	  strcat(word, " Heures");
	}
	
	if (strcmp(hourWord, "Midi ") &&
		strcmp(hourWord, "Minuit "))
		strcat(word, " Moins ");
	else
		strcat(word, "Moins ");
	
	Number2WordFrench(minutes, minuteWord);
	strcat(word, minuteWord);
  }
  else{
	strcat(word, hourWord);
	if (!strcmp(hourWord, "Une")){
	  strcat(word, " Heure ");
	}
	else if (strcmp(hourWord, "Midi ") &&
			 strcmp(hourWord, "Minuit ")){
	  strcat(word, " Heures ");
	}
	strcat(word, minuteWord);
  }
  
  return TRUE;
}

/*****************************************************************************
  Afrikaans Time Routines by Linda Matthews <linder@ibm.com>
 *****************************************************************************/

/*
  Converts a number from 0 to 59 into an afrikaans string equivalent
*/
Number2WordAfrikaans(number, word)
int number;
char *word;
{
  static char *underTwenty[] = {
	"Nul", "Een", "Twee", "Drie", "Vier", "Vyf", "Ses", "Sewe",
	"Agt", "Nege", "Tien", "Elf", "Twaalf", "Dertien", "Veertien",
	"Vyftien", "Sestien", "Sewentien", "Agtien", "Neentien"
	  };
  static char *overNineteen[] = {
	"", "Tien", "Twintig", "Dertig", "Veertig", "Vyftig"
	  };
  int guzzinto, remainder;
  
  *word = '\0';

  if ((number < 0) || (number > 59)){
	return FALSE;
  }
  
  if (number < 20){
	strcat(word, underTwenty[number]);
	return TRUE;
  }
  
  guzzinto = number/10;
  remainder = number%10;
  
  if(remainder != 0){
	strcat(word, underTwenty[remainder]);
	strcat(word, "-en-");
  }
  strcat(word, overNineteen[guzzinto]);
  
  return TRUE;
}

/*
  Turns 24 hour time (with minutes) into afrikaans representation
*/
Time2WordAfrikaans(hour, minutes, word)
int hour, minutes;
char *word;
{
  int pm, rounded = 0;
  char hourWord[MAX_TIMESTRING], minuteWord[MAX_TIMESTRING];
  
  *word = '\0';
  
  pm = (hour > 11);
  
  if (pm) {
	hour -= 12;
  }
  if (hour == 0){
	hour = 12;
  }
  
  if(approximate){
	if((rounded = minutes%5) >= 3){
	  minutes += (5 - rounded);
	  if(minutes == 60){
		minutes = 0;
		TimeIncrement(&hour, &pm);
	  }
	}
	else if(rounded > 0){
	  minutes -= rounded;
	}
	
	strcat(word, rounded?"Ongeveer ":"");
  }
  
  Number2WordAfrikaans(hour, hourWord);
  Number2WordAfrikaans(minutes, minuteWord);
  
  /*
	Try to do some cute things with the time
  */
  if(minutes == 0){
	if(hour == 12){
	  strcat(word, pm?"Middag":"Middernag");
	  return TRUE;
	}
	else{
	  strcat(word, hourWord);
	  strcat(word, " Uur");
	}
  }
  else if((minutes >= 1) && (minutes <= 9)){
	strcat(word, minuteWord);
	strcat(word, " Minute Oor ");
	strcat(word, hourWord);
  }
  else if(minutes == 15){
	strcat(word, "Kwart Oor ");
	strcat(word, hourWord);
  }
  else if(minutes == 30){
	strcat(word, "Half ");
	TimeIncrement(&hour, &pm);
	Number2WordAfrikaans(hour, hourWord);
	strcat(word, hourWord);
  }
  else if(minutes == 45){
	strcat(word, "Kwart Voor ");
	TimeIncrement(&hour, &pm);
	Number2WordAfrikaans(hour, hourWord);
	strcat(word, hourWord);
  }
  else if((minutes > 30) && before){
	TimeIncrement(&hour, &pm);
	minutes = 60 - minutes;
	Number2WordAfrikaans(hour, hourWord);
	Number2WordAfrikaans(minutes, minuteWord);
	strcat(word, minuteWord);
	if(minutes%5){
	  strcat(word, " Minute");
	}
	strcat(word, " Voor ");
	strcat(word, hourWord);
  }
  else{
	strcat(word, hourWord);
	strcat(word, " ");
	strcat(word, minuteWord);
  }
  if(!noampm){
	strcat(word, pm?" nm":" vm");
  }
  
  return TRUE;
}

/*****************************************************************************
  Dutch Time Routines by Richard van de Stadt <stadt@cs.utwente.nl>
 *****************************************************************************/

/*
  Converts a number from 0 to 59 into an dutch string equivalent
*/
Number2WordDutch(number, word)
int number;
char *word;
{
  static char *underTwenty[] = {
	"nul", "een", "twee", "drie", "vier", "vijf", "zes", "zeven",
	"acht", "negen", "tien", "elf", "twaalf", "dertien", "veertien",
	"vijftien", "zestien", "zeventien", "achttien", "negentien"
	  };
  static char *overNineteen[] = {
	"", "tien", "twintig", "dertig", "veertig", "vijftig"
	  };
  int guzzinto, remainder;
  
  *word = '\0';

  if ((number < 0) || (number > 59)){
	return FALSE;
  }
  
  if (number < 20){
	strcat(word, underTwenty[number]);
	return TRUE;
  }
  
  guzzinto = number/10;
  remainder = number%10;
  
  if(remainder != 0){
	strcat(word, underTwenty[remainder]);
	strcat(word, "en");
  }
  strcat(word, overNineteen[guzzinto]);
  
  return TRUE;
}

/*
  Turns 24 hour time (with minutes) into dutch representation
*/
Time2WordDutch(hour, minutes, word)
int hour, minutes;
char *word;
{
  int pm, rounded = 0, savehour = hour;
  char hourWord[MAX_TIMESTRING], minuteWord[MAX_TIMESTRING];
  
  *word = '\0';
  
  pm = (hour > 11);
  
  if (pm) {
	hour -= 12;
  }
  if (hour == 0){
	hour = 12;
  }
  
  if(approximate){
	if((rounded = minutes%5) >= 3){
	  minutes += (5 - rounded);
	  if(minutes == 60){
		minutes = 0;
		TimeIncrement(&hour, &pm);
	  }
	}
	else if(rounded > 0){
	  minutes -= rounded;
	}
	
	strcat(word, rounded?"ongeveer ":"");
  }
  
  Number2WordDutch(hour, hourWord);
  Number2WordDutch(minutes, minuteWord);
  
  /*
	Try to do some cute things with the time
  */
  if(minutes == 0){
  	strcat(word, hourWord);
  	strcat(word, " uur");
  }
  else if(minutes == 15){
	strcat(word, "kwart over ");
	strcat(word, hourWord);
  }
  else if((minutes == 20) && half){
	strcat(word, minuteWord);
	strcat(word, " over ");
  	strcat(word, hourWord);
  }
  else if((minutes >= 1) && (minutes <= 19)){
	strcat(word, minuteWord);
	strcat(word, " over ");
	strcat(word, hourWord);
  }
  else if((minutes >= 20) && (minutes <=29)){
	TimeIncrement(&hour, &pm);
	minutes = 30 - minutes;
	Number2WordDutch(hour, hourWord);
	Number2WordDutch(minutes, minuteWord);
	strcat(word, minuteWord);
	strcat(word, " voor half ");
	strcat(word, hourWord);
  }
  else if(minutes == 30){
	strcat(word, "half ");
	TimeIncrement(&hour, &pm);
	Number2WordDutch(hour, hourWord);
	strcat(word, hourWord);
  }
  else if(minutes == 45){
	strcat(word, "kwart voor ");
	TimeIncrement(&hour, &pm);
	Number2WordDutch(hour, hourWord);
	strcat(word, hourWord);
  }
  else if((minutes >= 40) && (minutes <= 44) && before){
	TimeIncrement(&hour, &pm);
	minutes = 60 - minutes;
	Number2WordDutch(hour, hourWord);
	Number2WordDutch(minutes, minuteWord);
	strcat(word, minuteWord);
	strcat(word, " voor ");
	strcat(word, hourWord);
  }
  else if((minutes >= 31) && (minutes <=44)){
	TimeIncrement(&hour, &pm);
	minutes -= 30;
	Number2WordDutch(hour, hourWord);
	Number2WordDutch(minutes, minuteWord);
	strcat(word, minuteWord);
	strcat(word, " over half ");
	strcat(word, hourWord);
  }
  else if((minutes >= 46) && (minutes <=59)){
	TimeIncrement(&hour, &pm);
	minutes = 60 - minutes;
	Number2WordDutch(hour, hourWord);
	Number2WordDutch(minutes, minuteWord);
	strcat(word, minuteWord);
	strcat(word, " voor ");
	strcat(word, hourWord);
  }
/* No other possibilities
  else{
	strcat(word, hourWord);
	strcat(word, " ");
	strcat(word, minuteWord);
  }
*/
  if(!noampm){
	if ((savehour >= 0) && (savehour < 6)) {
	  strcat (word, " 's nachts");
	}
	else if  ((savehour >= 6) && (savehour < 12)) {
	  strcat (word, " 's ochtends");
	}
	else if  ((savehour >= 12) && (savehour < 18)) {
	  strcat (word, " 's middags");
	}
	else {
	  strcat (word, " 's avonds");
	}
	/*
	strcat(word, savepm?" nm":" vm");
	*/
  }
  return TRUE;
}


/*****************************************************************************
  Finnish Time Routines by Vesa K|pp{ <k103063@cc.tut.fi>
 *****************************************************************************/

/*
  Converts a number from 0 to 59 into an finnish string equivalent
*/
Number2WordFinnish(number, word)
int number;
char *word;
{
  static char *underTwenty[] = {
	"Nolla", "Yksi", "Kaksi", "Kolme", "Nelja", "Viisi", "Kuusi",
	"Seitseman", "Kahdeksan", "Yhdeksan", "Kymmenen", "Yksitoista",
	"Kaksitoista", "Kolmetoista", "Neljatoista", "Viisitoista",
	"Kuusitoista", "Seitsemantoista", "Kahdeksantoista", "Yhdeksantoista"
	  };
  static char *overNineteen[] = {
	"", "toista", "Kaksikymmenta", "Kolmekymmenta", "Neljakymmenta",
	"Viisikymmenta"
	  };
  int guzzinto, remainder;
  
  *word = '\0';

  if ((number < 0) || (number > 59)){
	return FALSE;
  }
  
  if (number < 20){
	strcat(word, underTwenty[number]);
	return TRUE;
  }
  
  guzzinto = number/10;
  remainder = number%10;
  
  strcat(word, overNineteen[guzzinto]);
  
  if(remainder != 0){
	strcat(word, underTwenty[remainder]);
  }
  
  return TRUE;
}

/*
  Turns 24 hour time (with minutes) into finnish representation
*/
Time2WordFinnish(hour, minutes, word)
int hour, minutes;
char *word;
{
  int pm, rounded = 0;
  char hourWord[MAX_TIMESTRING], minuteWord[MAX_TIMESTRING];
  
  *word = '\0';
  
  pm = (hour > 11);
  
  if (pm) {
	hour -= 12;
  }
  if (hour == 0){
	hour = 12;
  }
  
  /* Round to nearest 5 minutes */
  if(approximate){
	if((rounded = minutes%5) >= 3){
	  minutes += (5 - rounded);
	  /* If round to next hour, handle correctly */
	  if(minutes == 60){
		minutes = 0;
		TimeIncrement(&hour, &pm);
	  }
	}
	else if(rounded > 0){
	  minutes -= rounded;
	}
	
	strcat(word, rounded?"Noin ":"");
  }
  
  Number2WordFinnish(hour, hourWord);
  Number2WordFinnish(minutes, minuteWord);
  
  /*
	Try to do some cute things with the time
	*/
  if(minutes == 0){
	if(hour == 12){
	  strcat(word, pm?"Puolipaiva":"Keskiyo");
	  return TRUE;
	}
	else{
	  strcat(word, "Tasan ");
	  strcat(word, hourWord);
	}
  }
  else if((minutes >= 1) && (minutes <= 29)){
	strcat(word, minuteWord);
	strcat(word, " yli ");
	strcat(word, hourWord);
  }
  else if(minutes == 15){
	strcat(word, "Vartin yli ");
	strcat(word, hourWord);
  }
  else if(minutes == 30){
	strcat(word, "Puoli ");
	TimeIncrement(&hour, &pm);
	Number2WordFinnish(hour, hourWord);
	strcat(word, hourWord);
  }
  else if(minutes == 45){
	strcat(word, "Varttia vaille ");
	TimeIncrement(&hour, &pm);
	Number2WordFinnish(hour, hourWord);
	strcat(word, hourWord);
  }
  else if((minutes > 30) && before){
	TimeIncrement(&hour, &pm);
	minutes = 60 - minutes;
	Number2WordFinnish(hour, hourWord);
	Number2WordFinnish(minutes, minuteWord);
	strcat(word, minuteWord);
	strcat(word, " vaille ");
	strcat(word, hourWord);
  }
  else{
	strcat(word, hourWord);
	strcat(word, " ");
	strcat(word, minuteWord);
  }

  if(!noampm){
	strcat(word, pm?" pm":" am");
  }
  
  return TRUE;
}

/*****************************************************************************
  Pig Latin Time Routines by badger@inmet.camb.inmet.com (John 'Pig' Reardon)
 *****************************************************************************/

/*
  Converts a number from 0 to 59 into an pig-latin string equivalent
*/
Number2WordPigLatin(number, word)
int number;
char *word;
{
  static char *underTwenty[] = {
	"Ero-Zay", "One-Yay", "Oo-Tay", "ee-Thray", "Or-Fay", "Ive-Fay", 
	"Ix-Say", "Even-Say", "Eight-Yay", "Ine-Nay", "En-Tay",
	"Eleven-Yay", "Elve-Tway", "Irteen-Thay", "Orteen-Fay",
	"Ifteen-Fay", "Ixteen-Say", "Eventeen-Say", "Eighteen-Yay",
	"Ineteen-Nay"
	};
  static char *overNineteen[] = {
	"", "Een-Tay", "Enty-Tway", "Irty-Thay", "Orty-Fay", "Ifty-Fay"
	  };
  int guzzinto, remainder;
  
  *word = '\0';

  if ((number < 0) || (number > 59)){
	return FALSE;
  }
  
  if (number < 20){
	strcat(word, underTwenty[number]);
	return TRUE;
  }
  
  guzzinto = number/10;
  remainder = number%10;
  
  strcat(word, overNineteen[guzzinto]);
  
  if(remainder != 0){
	strcat(word, " ");
	strcat(word, underTwenty[remainder]);
  }
  
  return TRUE;
}

/*
  Turns 24 hour time (with minutes) into pig latin representation
*/
Time2WordPigLatin(hour, minutes, word)
int hour, minutes;
char *word;
{
  int pm, rounded = 0;
  char hourWord[MAX_TIMESTRING], minuteWord[MAX_TIMESTRING];
  
  *word = '\0';
  
  pm = (hour > 11);
  
  if (pm) {
	hour -= 12;
  }
  if (hour == 0){
	hour = 12;
  }
  
  /* Round to nearest 5 minutes */
  if(approximate){
	if((rounded = minutes%5) >= 3){
	  minutes += (5 - rounded);
	  /* If round to next hour, handle correctly */
	  if(minutes == 60){
		minutes = 0;
		TimeIncrement(&hour, &pm);
	  }
	}
	else if(rounded > 0){
	  minutes -= rounded;
	}
	
	strcat(word, rounded?"About-Yay ":"");
  }
  
  Number2WordPigLatin(hour, hourWord);
  Number2WordPigLatin(minutes, minuteWord);
  
  /*
	Try to do some cute things with the time
	*/
  if(minutes == 0){
	if(hour == 12){
	  strcat(word, pm?"Oon-Nay":"Idnight-May");
	  return TRUE;
	}
	else{
	  strcat(word, hourWord);
	  strcat(word, " O-Yay'ock-Clay");
	}
  }
  else if((minutes >= 1) && (minutes <= 9)){
	strcat(word, minuteWord);
	strcat(word, " Ast-Pay ");
	strcat(word, hourWord);
  }
  else if(minutes == 15){
	strcat(word, "Orter-Quay Ast-Pay ");
	strcat(word, hourWord);
  }
  else if(minutes == 30){
	strcat(word, "Alf-Hay Ast-Pay ");
	strcat(word, hourWord);
  }
  else if(minutes == 45){
	strcat(word, "Orter-Quay Oo-Tay ");
	TimeIncrement(&hour, &pm);
	Number2WordPigLatin(hour, hourWord);
	strcat(word, hourWord);
  }
  else if((minutes > 30) && before){
	TimeIncrement(&hour, &pm);
	minutes = 60 - minutes;
	Number2WordPigLatin(hour, hourWord);
	Number2WordPigLatin(minutes, minuteWord);
	strcat(word, minuteWord);
	strcat(word, " Oo-Tay ");
	strcat(word, hourWord);
  }
  else{
	strcat(word, hourWord);
	strcat(word, " ");
	strcat(word, minuteWord);
  }

  if(!noampm){
	strcat(word, pm?" ee-em-pay":" ay-em-yay");
  }
  
  return TRUE;
}

/*****************************************************************************
  Spanish Time Routines by Chris Hadley	<ckh@cl.cam.ac.uk>
 *****************************************************************************/

/*
  Converts a number from 0 to 59 into an spanish string equivalent
*/
Number2WordSpanish(number, word)
int number;
char *word;
{
  /* use "una" as word understood, but not expressed, is "hora" */

  static char *underThirty[] = {
	"cero", "una", "dos", "tres", "cuatro", "cinco", "seis", "siete",
	"ocho", "nueve", "diez", "once", "doce", "trece", "catorce",
	"quince", "dieciseis", "diecisiete", "dieciocho", "diecinueve",
	"veinte", "veintiuna", "veintidos", "veintitres", "veinticuatro", 
	"veinticinco", "veintiseis", "veintisiete", "veintiocho", "veintinueve"	  
	};
  static char *overTwentynine[] = {
	"", "", "", "treinta", "cuarenta", "cincuenta"
	  };
  int guzzinto, remainder;
  
  *word = '\0';

  if ((number < 0) || (number > 59)){
	return FALSE;
  }
  
  if (number < 30){
	strcat(word, underThirty[number]);
	return TRUE;
  }
  
  guzzinto = number/10;
  remainder = number%10;
  
  strcat(word, overTwentynine[guzzinto]);
  
  if(remainder != 0){
	strcat(word, " y ");
	strcat(word, underThirty[remainder]);
  }
  
  return TRUE;
}

/*
  Turns 24 hour time (with minutes) into spanish representation
*/
Time2WordSpanish(hour, minutes, word)
int hour, minutes;
char *word;
{
  int tarde; 
  int rounded = 0;
  char hourWord[MAX_TIMESTRING], minuteWord[MAX_TIMESTRING];
  char bit_of_day[MAX_TIMESTRING];
  *word = '\0';
  
  
  /* use 1am - 12am = manana
	 12am - 2pm = mediodia
	 2pm - 8pm = tarde
	 8pm - 1am = noche
	 This varies greatly among spanish-speaking countries, but this
	 is about right for Spain itself. Don't bother with "madrugada"
	 (= early morning) as it is seldom used.
	 */
  if(!noampm){
	if(hour >= 1 && hour < 12) sprintf(bit_of_day," de la manana");
	if(hour == 12 && hour < 14) sprintf(bit_of_day," del mediodia");
	if(hour >=14 && hour < 20)   sprintf(bit_of_day," de la tarde");
	if((hour == 0) || (hour >=20 && hour <= 24))
		sprintf(bit_of_day," de la noche");
  }
  
  tarde = (hour > 11);
  
  if (tarde) {
	hour -= 12;
  }
  if (hour == 0){
	hour = 12;
  }
  
  /* Round to nearest 5 minutes */
  if(approximate){
	if((rounded = minutes%5) >= 3){
	  minutes += (5 - rounded);
	  /* If round to next hour, handle correctly */
	  if(minutes == 60){
		minutes = 0;
		TimeIncrement(&hour, &tarde);
	  }
	}
	else if(rounded > 0){
	  minutes -= rounded;
	}
	
	strcat(word, rounded?"Aproximadamente ":"");
  }
  
  Number2WordSpanish(hour, hourWord);
  Number2WordSpanish(minutes, minuteWord);
  
  /*
	Try to do some cute things with the time
	*/
  
  if(minutes == 0){
	if(hour == 12){
	  strcat(word, tarde?"La Mediodia":"La Medianoche");
	  return TRUE;
	}else{         
	  strcat(word, (hour == 1)?"La ":"Las "); 
	  strcat(word, hourWord);
	}
  }
  else if((minutes >= 1) && (minutes <= 9)){
	strcat(word, (hour == 1)?"La ":"Las "); 
	strcat(word, hourWord);
	strcat(word, " y ");
	strcat(word, minuteWord);
  }
  else if(minutes == 15){
	strcat(word, (hour == 1)?"La ":"Las "); 
	strcat(word, hourWord);
	strcat(word, " y cuarto");
  }
  else if(minutes == 30){
	strcat(word, (hour == 1)?"La ":"Las "); 
	strcat(word, hourWord);
	strcat(word, " y media");
  }
  else if(minutes == 45){
	TimeIncrement(&hour, &tarde);
	Number2WordSpanish(hour, hourWord);
	strcat(word, (hour == 1)?"La ":"Las "); 
	strcat(word, hourWord);
	strcat(word, " menos cuarto");
  }
  else if((minutes > 30) && before){
	TimeIncrement(&hour, &tarde);
	minutes = 60 - minutes;
	Number2WordSpanish(hour, hourWord);
	Number2WordSpanish(minutes, minuteWord);
	strcat(word, (hour == 1)?"La ":"Las "); 
	strcat(word, hourWord);
	strcat(word, " menos ");
	strcat(word, minuteWord);
  }
  else{
	strcat(word, (hour == 1)?"La ":"Las "); 
	strcat(word, hourWord);
	strcat(word, " y ");
	strcat(word, minuteWord);
  }
  
  if(!noampm){
	strcat(word,bit_of_day);
  }
  
  return TRUE;
}

/*****************************************************************************
  Japanese Time Routines by Johan van der Werf <jvdwerf@mmra1.ms.philips.nl>
 *****************************************************************************/

/*
  Converts a number from 0 to 59 into an japanese string equivalent
*/
Number2MinuteJapanese(number, word)
int number;
char *word;
{
  int guzzinto, remainder;
  static char* tens[] = {
    "", "juu-", "nijuu-", "sanjuu-", "yonjuu-", "gojuu-"
	  };
  static char* fulltens[] = {
    "", "juppun", "nijuppun", "sanjuppun", "yonjuppun", "gojuppun"
	  };
  static char* mins[] = {
    "", "ippun", "nifun", "sanpun", "yonpun", "gofun",
    "roppun", "shichifun", "happun", "kyuufun"
	  };
  
  *word = '\0';
  
  if ((number < 0) || (number > 59)){
	return FALSE;
  }
  
  guzzinto = number/10;
  remainder = number%10;
  
  if(remainder != 0){
    strcat(word, tens[guzzinto]);
    strcat(word, mins[remainder]);
  }
  else {
    strcat(word, fulltens[guzzinto]);
  }
  return TRUE;
}

/*
  Turns 24 hour time (with minutes) into japanese representation
*/
Time2WordJapanese(hour, minutes, word)
int hour, minutes;
char *word;
{
  int pm, rounded = 0;
  char minuteWord[MAX_TIMESTRING];
  static char* ji[] = {
    "nani ???", "ichiji", "niji", "sanji", "yoji", "goji", "rokuji",
    "shichiji", "hachiji", "kuji", "juuji", "juuichiji", "juuniji"
	  };
  *word = '\0';
  
  pm = (hour > 11);
  
  if (pm) {
	hour -= 12;
  }
  if (hour == 0){
	hour = 12;
  }
  
  /* Round to nearest 5 minutes */
  if(approximate){
	if((rounded = minutes%5) >= 3){
	  minutes += (5 - rounded);
	  /* If round to next hour, handle correctly */
	  if(minutes == 60){
		minutes = 0;
		TimeIncrement(&hour, &pm);
	  }
	}
	else if(rounded > 0){
	  minutes -= rounded;
	}
  }
  
  Number2MinuteJapanese(minutes, minuteWord);
  
  /*
	Try to do some cute things with the time
	*/
  if(minutes == 0){
	if(hour == 12){
	  strcat(word, pm?"hiru":"mayonaka");
	  
	  /* If rounded to nearest 5 minutes */
	  if(approximate)
		  strcat(word, rounded?" goro":"");
	  
	  return TRUE;
	}
  }
  
  if(!noampm){
	strcat(word, pm?"gogo ":"gozen ");
  }
  
  if((minutes >= 1) && (minutes <= 29)){
	strcat(word, ji[hour]);
	strcat(word, " ");
	strcat(word, minuteWord);
  }
  else if(minutes == 30){
	strcat(word, ji[hour]);
	strcat(word, "han");
  }
  else if(minutes == 45){
	TimeIncrement(&hour, &pm);
	strcat(word, ji[hour]);
	strcat(word, " juugofun mae");
  }
  else if((minutes > 30) && before){
	TimeIncrement(&hour, &pm);
	minutes = 60 - minutes;
	strcat(word, ji[hour]);
	strcat(word, " ");
	Number2MinuteJapanese(minutes, minuteWord);
	strcat(word, minuteWord);
	strcat(word, " mae");
  }
  else{
	strcat(word, ji[hour]);
	strcat(word, " ");
	strcat(word, minuteWord);
  }
  
  /* If rounded to nearest 5 minutes */
  if(approximate){
	strcat(word, rounded?" goro":"");
  }
  return TRUE;
}

/*****************************************************************************
  Kanji Time Routines by Johan van der Werf <jlinz@krokus.han.de>
 *****************************************************************************/

/*
  Converts a number from 0 to 59 into an kanji string equivalent
*/
Number2WordKanji(number, word)
int number;
char *word;
{
  int guzzinto, remainder;
  static char* num[] = {
    "\033$B2?\033(J", "\033$B0l\033(J", "\033$BFs\033(J",
    "\033$B;0\033(J", "\033$B;M\033(J", "\033$B8^\033(J",
    "\033$BO;\033(J", "\033$B<7\033(J", "\033$BH,\033(J",
    "\033$B6e\033(J"
	  };
  
  *word = '\0';
  
  if ((number < 0) || (number > 59)){
	return FALSE;
  }
  
  guzzinto = number/10;
  remainder = number%10;
  
  if (guzzinto != 0) {
    if (guzzinto != 1)
		strcat(word, num[guzzinto]);
    strcat(word, "\033$B==\033(J");
  }
  if (remainder != 0) {
    strcat(word, num[remainder]);
  }
  
  return TRUE;
}

/*
  Turns 24 hour time (with minutes) into kanji representation
*/
Time2WordKanji(hour, minutes, word)
int hour, minutes;
char *word;
{
  int pm, rounded = 0;
  char hourWord[MAX_TIMESTRING], minuteWord[MAX_TIMESTRING];
  
  *word = '\0';
  
  pm = (hour > 11);
  
  if (pm) {
	hour -= 12;
  }
  if (hour == 0){
	hour = 12;
  }
  
  /* Round to nearest 5 minutes */
  if(approximate){
	if((rounded = minutes%5) >= 3){
	  minutes += (5 - rounded);
	  /* If round to next hour, handle correctly */
	  if(minutes == 60){
		minutes = 0;
		TimeIncrement(&hour, &pm);
	  }
	}
	else if(rounded > 0){
	  minutes -= rounded;
	}
  }
  
  Number2WordKanji(hour, hourWord);
  Number2WordKanji(minutes, minuteWord);
  
  /*
	Try to do some cute things with the time
	*/
  if(minutes == 0){
	if(hour == 12){
	  strcat(word, pm?"\033$B@58a\033(J":"\033$B??LkCf\033(J");
	  
	  /* If rounded to nearest 5 minutes */
	  if(approximate)
		  strcat(word, rounded?" \033$B$4$m\033(J":"");
	  
	  return TRUE;
	}
  }
  
  if(!noampm){
	strcat(word, pm?"\033$B8a8e\033(J ":"\033$B8aA0\033(J ");
  }
  
  if(minutes == 0){				/* MAKOTO */
	strcat(word, hourWord);
	strcat(word, "\033$B;~\033(J");
  }
  else if(minutes == 30){
	strcat(word, hourWord);
	strcat(word, "\033$B;~H>\033(J");
  }
  else if(minutes == 45){
	TimeIncrement(&hour, &pm);
	Number2WordKanji(hour, hourWord);
	strcat(word, hourWord);
	strcat(word, "\033$B;~==8^J,A0\033(J");
  }
  else if((minutes > 30) && before){
	TimeIncrement(&hour, &pm);
	Number2WordKanji(hour, hourWord);
	minutes = 60 - minutes;
	strcat(word, hourWord);
	strcat(word, "\033$B;~\033(J");
	Number2WordKanji(minutes, minuteWord);
	strcat(word, minuteWord);
	strcat(word, "\033$BJ,A0\033(J");
  }
  else{
	strcat(word, hourWord);
	strcat(word, "\033$B;~\033(J");
	strcat(word, minuteWord);
	strcat(word, "\033$BJ,\033(J");
  }
  
  /* If rounded to nearest 5 minutes */
  if(approximate){
	strcat(word, rounded?" \033$B$4$m\033(J":"");
  }
  return TRUE;
}

/*****************************************************************************
       German Time Routines by Jochen Linz <jlinz@krokus.han.de>
*****************************************************************************/

int Number2WordGerman(number, word)
int number;
char *word;
{
  static char *underTwenty[] = {
	"null", "ein", "zwei", "drei", "vier", "fuenf", "sechs", "sieben",
	"acht", "neun", "zehn", "elf", "zwoelf", "dreizehn", "vierzehn",
	"fuenfzehn", "sechzehn", "siebzehn", "achtzehn", "neunzehn"
	  };
  static char *overNineteen[] = {
	"", "", "zwanzig", "dreissig", "vierzig", "fuenfzig"
	  };
  int guzzinto, remainder;
  
  *word = '\0';
  
  if ((number < 0) || (number > 59)){
	return FALSE;
  }
  
  if (number < 20){
	strcpy(word, underTwenty[number]);
	return TRUE;
  }
  
  guzzinto = number/10;
  remainder = number%10;
  
  if(remainder != 0){
	strcpy(word,underTwenty[remainder]);
	strcat(word,"und");
	strcat(word,overNineteen[guzzinto]);
  }
  else{
	strcpy(word, overNineteen[guzzinto]);
  }
  
  return TRUE;
}

/*
  Turns 24 hour time (with minutes) into German representation
  */
int Time2WordGerman(hour, minutes, word)
int hour, minutes; 
char *word;
{
  int rounded = 0;
  char hourWord[MAX_TIMESTRING], minuteWord[MAX_TIMESTRING];
  char dayWord[MAX_TIMESTRING]; /* special for german version */
  
  /* leave hour in 24 hour format; it's often also the GERMAN way! */
  
  *word = '\0';
  
  if(approximate){
	if((rounded = minutes%5) >= 3){
	  minutes += (5 - rounded);
	  if(minutes == 60){
		minutes = 0;
		hour = (hour+1)%24;
	  }
	}
	else if(rounded > 0){
	  minutes -= rounded;
	}
	
	strcat(word, rounded?"Etwa ":"");
  }
  
  if( hour >= 0 && hour < 12 )
	  strcpy(dayWord," morgens");
  else
	  if( hour >= 12 && hour < 15 )
		  strcpy(dayWord," nachmittags");
	  else
		  strcpy(dayWord," abends");
  
  if( minutes == 15 || minutes == 30 || minutes == 45 ){
	if(hour >= 12 )
		hour-=12;
	if(minutes == 15 ){
	  if(hour==0)
		  hour=12;
	  strcat(word,"Viertel nach ");
	}
	else{
	  hour+=1;			  /* the -before flag is a german rule */
	  if(minutes == 30 )
		  strcat(word,"Halb ");
	  else
		  strcat(word,"Viertel vor ");
	}
	if(hour == 1)
		strcat(word,"Eins");
	else{
	  Number2WordGerman(hour,hourWord);
	  hourWord[0]=toupper((int)hourWord[0]);
	  strcat(word,hourWord);
	}
	strcat(word,dayWord);
  }
  else{
	Number2WordGerman(hour,hourWord);
	Number2WordGerman(minutes,minuteWord);
	
	hourWord[0]=toupper((int)hourWord[0]); /* we're heavy casesensitive ;-) */
	
	strcat(word,hourWord);
	strcat(word," Uhr");
	if(minutes == 0){
	  if(hour > 0 && hour < 12 )
		  strcat(word, dayWord);
	  else
		  if(hour == 12 )
			  strcpy(word,"Mittag");
		  else
			  if( hour == 0 )
				  strcpy(word,"Mitternacht");
	}
	else{
	  strcat(word," ");
	  if(minutes == 1)
		  strcat(word,"eins");
	  else
		  strcat(word,minuteWord);
	}
  }
  
  
  return TRUE;
}

/*****************************************************************************
  Swedish Time Routines by H}kan Lennest}l <hakanl@lulea.trab.se>
 *****************************************************************************/

/*
  Converts a number from 0 to 59 into an swedish string equivalent
*/
Number2WordSwedish(number, word)
int number;
char *word;
{
  static char *underTwenty[] = {
	"Noll", "Ett", "Tv}", "Tre", "Fyra", "Fem", "Sex", "Sju",
	"]tta", "Nio", "Tio", "Elva", "Tolv", "Tretton", "Fjorton",
	"Femton", "Sexton", "Sjutton", "Arton", "Nitton"
	  };
  static char *overNineteen[] = {
	"Noll", "", "Tjugo", "Trettio", "Fyrtio", "Femtio"
	  };
  int guzzinto, remainder;
  
  *word = '\0';
  
  if ((number < 0) || (number > 59)){
	return FALSE;
  }
  
  if ((number >= 10) && (number <= 19)) {
	strcat(word, underTwenty[number]);
	return;
  }
  
  guzzinto = number/10;
  remainder = number%10;
  
  if ((! approximate) || (guzzinto != 0))
	  strcat(word, overNineteen[guzzinto]);
  
  if ((! approximate) || (remainder != 0)) {
	if (guzzinto == 0) {
	  if (! approximate)
		  strcat(word, "-");
	  strcat(word, underTwenty[remainder]);
	}
	else {
	  int l= strlen(word);
	  if (remainder != 0) {
		strcat(word, underTwenty[remainder]);
		word[l]= word[l] + 32;
	  }
	}
  }
  
  return TRUE;
}

void Convert8BitSwedish(str)
char* str;
{
  int len= strlen(str);
  int ind;
  char c;
  for (ind= 0; ind < len; ind++) {
	c= str[ind];	
	switch (c) {
	case '{' :
		c= 0344;
		break;
	  case '}':
		c= 0345;
		break;
	  case '|':
		c= 0366;
		break;
	  case '[':
		c= 0304;
	  case ']':
		c= 0305;
		break;
	  case '\\':
		c= 0326;
		break;
	  default:;
	  }
	str[ind] = c;
  }
}

/*
  Turns 24 hour time (with minutes) into Swedish representation
  */
Time2WordSwedish(hour, minutes, word)
int hour, minutes;
char *word;
{
  int pm, nexthour, hour12, nexthour12, rounded = 0;
  char hourWord[MAX_TIMESTRING], minuteWord[MAX_TIMESTRING];
  
  /* leave hour in 24 hour format; it's the swedish way! */
  /* but only for exact time; if approximate is selected, then use 12 hour */
  
  *word = '\0';
  *hourWord = '\0';
  *minuteWord = '\0';
  
  if(approximate){
	if((rounded = minutes%5) >= 3){
	  minutes += (5 - rounded);
	  if(minutes == 60){
		minutes = 0;
		hour = (hour+1)%24;
	  }
	}
	else if(rounded > 0){
	  minutes -= rounded;
	}
  }
  
  nexthour = (hour+1)%24;
  
  if (approximate && (hour > 12)) {
    hour12 = hour-12;
    nexthour12 = nexthour-12;
  }
  else {
    hour12 = hour;
    nexthour12 = nexthour;
    if (approximate && (nexthour12 > 12))
        nexthour12= nexthour12-12;
  }
  
  if (hour12 == 0)
	  hour12= 12;
  if (nexthour12 == 0)
	  nexthour12= 12;
  if (nexthour12 < 0)
	  nexthour12= 12;
  
  if (approximate) {
	if ((hour == 0) && (minutes == 0)) {
	  strcat(word, "Midnatt");
	  return TRUE;
	}
 	Number2WordSwedish(hour12, hourWord);
	if (minutes == 0) {
	  strcat(word, hourWord);
	}
	else if (minutes < 30) {
	  Number2WordSwedish(minutes, minuteWord);
	  if (minutes == 15)
		  strcpy(word, "Kvart");
	  else
		  strcpy(word, minuteWord);
	  strcat(word, " |ver ");
	  strcat(word, hourWord);
	}
	else if (minutes > 30) {
	  Number2WordSwedish(nexthour12, hourWord);
	  minutes= 30 - (minutes - 30);
	  Number2WordSwedish(minutes, minuteWord);
	  if (minutes == 15)
		  strcpy(word, "Kvart");
	  else
		  strcpy(word, minuteWord);
	  strcat(word, " i ");
	  strcat(word, hourWord);
	}
	else {
	  Number2WordSwedish(nexthour12, hourWord);
	  strcpy(word, "Halv ");
	  strcat(word, hourWord);
	}
  }
  else { 
	if ((hour == 0) && (minutes == 0)) {
	  strcpy(word, "Tjugofyra Noll-Noll");
	  return TRUE;
	}
  	Number2WordSwedish(hour, hourWord);
  	Number2WordSwedish(minutes, minuteWord);
	
  	strcpy(word, hourWord);
  	strcat(word, " ");
  	strcat(word, minuteWord);
  }
  if(latin1){
	Convert8BitSwedish(word);
  }
  return TRUE;
}

/*****************************************************************************
  Norwegian Time Routines by Ole Arntzen (olea@ii.uib.no)
 *****************************************************************************/

void Convert8BitNorwegian(str)
char* str;
{
  int len= strlen(str);
  int ind;
  char c;
  for (ind= 0; ind < len; ind++) {
        c= str[ind];
        switch (c) {
        case '{' :
                c= 0346;
                break;
          case '}':
                c= 0345;
                break;
          case '|':
                c= 0370;
                break;
          case '[':
                c= 0306;
          case ']':
                c= 0305;
                break;
          case '\\':
                c= 0330;
                break;
          default:;
          }
        str[ind] = c;
  }
}

Number2WordNorwegian(number, word)
int number;
char *word;
{
  static char *underTwenty[] = {
    "null", "ett", "to", "tre", "fire", "fem", "seks", "syv",
    "}tte", "ni", "ti", "elleve", "tolv", "tretten", "fjorten",
    "femten", "seksten", "sytten", "atten", "nitten"
  };
  static char *overNineteen[] = {
    "null", "ti", "tjue", "tretti", "f|rti", "femti"
  };
  int guzzinto, remainder;

  *word = '\0';

  if (before||approximate) {
    if ((number < 1) || (number > 14)){
      return FALSE;
    }
    strcpy(word, underTwenty[number]);
    return TRUE;
  }
  else {
    if ((number < 0) || (number > 59)){
      return FALSE;
    }
    if ((number >= 10) && (number <= 19)) {
      strcat(word, underTwenty[number]);
      return;
    }

    guzzinto = number/10;
    remainder = number%10;
    strcat(word, overNineteen[guzzinto]);
    if (guzzinto == 0) strcat(word, "-");
    if ((guzzinto==0)||(remainder!=0)) strcat(word, underTwenty[remainder]);
  }
}

/*
  Turns 24 hour time (with minutes) into Norwegian representation
*/

Time2WordNorwegian(hour, minutes, word)
int hour, minutes;
char *word;
{
  int pm, rounded = 0;
  int originalHour;
  char hourWord[MAX_TIMESTRING], minuteWord[MAX_TIMESTRING];

  *word = '\0';

  if (before||approximate) {
    pm = (hour > 11);
    originalHour = hour;
    if (pm) {
      hour -= 12;
    }
    if (hour == 0){
      hour = 12;
    }

    /* Round to nearest 5 minutes */
    if(approximate){
      if((rounded = minutes%5) >= 3){
        minutes += (5 - rounded);
        /* If round to next hour, handle correctly */
        if(minutes == 60){
          minutes = 0;
          TimeIncrement(&hour, &pm);
        }
      }
      else if(rounded > 0){
        minutes -= rounded;
      }
      strcat(word, rounded?"omtrent ":"");
    }
    else {
      /* Have a more formal style if we don't approximate */
      strcat(word,"klokken er ");
    }

    /*
    Try to do some cute things with the time
    */
    if(minutes == 0){
      Number2WordNorwegian(hour, hourWord);
      strcat(word, hourWord);
      if((!approximate) && hour == 12){
        strcat(word, pm?", middag.":", midnatt.");

        /* Uppercase first letter and return */
        *word = *word - ( 'a' - 'A' );
        return TRUE;
      }
    }
    else if((minutes >= 1) && (minutes <= 14)){
      Number2WordNorwegian(minutes, minuteWord);
      strcat(word, minuteWord);
      if(!approximate) strcat(word, minutes==1?" minutt":" minutter");
      strcat(word, " over ");
      Number2WordNorwegian(hour, hourWord);
      strcat(word, hourWord);
    }
    else if(minutes == 15){
      strcat(word, "kvart over ");
      Number2WordNorwegian(hour, hourWord);
      strcat(word, hourWord);
    }
    else if((minutes >= 16) && (minutes <= 29)){
      minutes = 30 - minutes;
      Number2WordNorwegian(minutes, minuteWord);
      strcat(word, minuteWord);
      if(!approximate) strcat(word, minutes==1?" minutt":" minutter");
      strcat(word, " p} halv ");
      Number2WordNorwegian((hour%12)+1, hourWord);
      strcat(word, hourWord);
    }
    else if(minutes == 30){
      strcat(word, "halv ");
      Number2WordNorwegian((hour%12)+1, hourWord);
      strcat(word, hourWord);
    }
    else if((minutes >= 31) && (minutes <= 44)){
      minutes = minutes - 30;
      Number2WordNorwegian(minutes, minuteWord);
      strcat(word, minuteWord);
      if(!approximate) strcat(word, minutes==1?" minutt":" minutter");
      strcat(word, " over halv ");
      Number2WordNorwegian((hour%12)+1, hourWord);
      strcat(word, hourWord);
    }
    else if(minutes == 45){
      strcat(word, "kvart p} ");
      Number2WordNorwegian((hour%12)+1, hourWord);
      strcat(word, hourWord);
    }
    else if((minutes >= 46) && (minutes <= 59)){
      minutes = 60 - minutes;
      Number2WordNorwegian(minutes, minuteWord);
      strcat(word, minuteWord);
      if(!approximate) strcat(word, minutes==1?" minutt":" minutter");
      strcat(word, " p} ");
      Number2WordNorwegian((hour%12)+1, hourWord);
      strcat(word, hourWord);
    }
    else{
      fprintf(stderr,"Shouldn't be here, hour=%d, minutes=%d\n",hour,minutes);
    }

    /* Add part of day if we don't approximate */
    if(!approximate) {
      if(originalHour<6){
        strcat(word, " om natten.");
      }
      else if(originalHour<9){
        strcat(word, " om morgenen.");
      }
      else if(originalHour<12){
        strcat(word, " om formiddagen.");
      }
      else if(originalHour<18){
        strcat(word, " om ettermiddagen.");
      }
      else{
        strcat(word, " om kvelden.");
      }
    }
  } /* end if before */
  else {
    /* approximate has no meaning in this context */
    Number2WordNorwegian(hour,hourWord);
    strcat(word, hourWord);
    strcat(word, " ");
    Number2WordNorwegian(minutes, minuteWord);
    strcat(word, minuteWord);
  }

  /* Uppercase first letter */
  *word = *word - ( 'a' - 'A' );

  if(latin1){
    Convert8BitNorwegian(word);
  }
  return TRUE;
}


/*****************************************************************************
  VietNamese Time Routines by Kiem (V.K.) Ngo <KINGO@BNR.CA>
 *****************************************************************************/

Number2WordVietNamese(number, word)
int number;
char *word;
{
  static char *underTen[] = {
	"Khong", "Mot", "Hai", "Ba", "Bon", "Nam", "Sau", "Bay", "Tam", "Chin"
	 };
  static char *overNine[] = {
	"", "Muoi", "Hai muoi", "Ba muoi", "Bon muoi", "Nam muoi"
	  };

  int guzzinto, remainder;

  *word = '\0';

  if ((number < 0) || (number > 59)){
	return FALSE;
  }

  if (number < 10){
	strcat(word, underTen[number]);
	return TRUE;
  }

  guzzinto = number/10;
  remainder = number%10;

  strcat(word, overNine[guzzinto]);

  if(remainder != 0){
	strcat(word, " ");
	strcat(word, underTen[remainder]);
  }

  return TRUE;
}

/***********
    gio == hour
    phut == minute
    khoang == approximate
    dung ngo == noon
    nua dem == midnight
    ruoi == half
    kem == less
    chieu == pm
    sang == am
    ****************/
Time2WordVietNamese(hour, minutes, word)
int hour, minutes;
char *word;
{
  int pm, rounded = 0;
  char hourWord[MAX_TIMESTRING], minuteWord[MAX_TIMESTRING];

  *word = '\0';

  pm = (hour > 11);

  if (pm) {
	hour -= 12;
  }
  if (hour == 0){
	hour = 12;
  }

  /* Round to nearest 5 minutes */
  if(approximate){
	if((rounded = minutes%5) >= 3){
	  minutes += (5 - rounded);
	  /* If round to next hour, handle correctly */
	  if(minutes == 60){
		minutes = 0;
		TimeIncrement(&hour, &pm);
	  }
	}
	else if(rounded > 0){
	  minutes -= rounded;
	}
	
	strcat(word, rounded?"Khoang ":"");
  }

  Number2WordVietNamese(hour, hourWord);
  Number2WordVietNamese(minutes, minuteWord);

  if(minutes == 0){
	if(hour == 12){
	  strcat(word, pm?"Dung Ngo":"Nua Dem");
	  return TRUE;
	}
	else{
	  strcat(word, hourWord);
	  strcat(word, " gio dung");
	}
  }
  else if(minutes == 30){
	strcat(word, hourWord);
	strcat(word, " gio Ruoi ");
  }
  else if(minutes == 45){
	TimeIncrement(&hour, &pm);
	Number2WordVietNamese(hour, hourWord);
	strcat(word, hourWord);
	strcat(word, "gio Kem Muoi Nam ");
	strcat(word, " phut,");
  }
  else if((minutes > 30) && before){
	TimeIncrement(&hour, &pm);
	minutes = 60 - minutes;
	Number2WordVietNamese(hour, hourWord);
	Number2WordVietNamese(minutes, minuteWord);
	strcat(word, hourWord);
	strcat(word, " gio Kem ");
	strcat(word, minuteWord);
	strcat(word, " phut,");
  }
  else{
	strcat(word, hourWord);
	strcat(word, " gio, ");
	strcat(word, minuteWord);
	strcat(word, " phut,");
  }
  if(!noampm){
	strcat(word, pm?" Chieu":" Sang");
  }
  return TRUE;
}

/*****************************************************************************
  Italian Time Routines by Massimo Dal Zotto <dz@irst.it>
 *****************************************************************************/

/*
  Converts a number from 0 to 59 into an italian string equivalent
*/
Number2WordItalian(number, word)
int number;
char *word;
{
  static char *underThirty[] = {
	"zero", "uno", "due", "tre", "quattro", "cinque", "sei", "sette",
	"otto", "nove", "dieci", "undici", "dodici", "tredici", "quattordici",
	"quindici", "sedici", "diciassette", "diciotto", "diciannove",
	"venti", "ventuno", "ventidue", "ventitre", "ventiquattro", 
	"venticinque", "ventisei", "ventisette", "ventotto", "ventinove"
	  };
  static char *overTwentynine[] = {
	"", "", "", "trenta", "quaranta", "cinquanta"
	  };
  int guzzinto, remainder;
  
  *word = '\0';
  
  if ((number < 0) || (number > 59)){
	return FALSE;
  }
  
  if (number < 30){
	strcat(word, underThirty[number]);
	return TRUE;
  }
  
  guzzinto = number/10;
  remainder = number%10;
  
  strcat(word, overTwentynine[guzzinto]);
  
  if(remainder != 0){
	if ((underThirty[remainder][0] == 'u') ||
		(underThirty[remainder][0] == 'o'))
		word[strlen(word) - 1] = '\0';
	strcat(word, underThirty[remainder]);
  }
  
  return TRUE;
}

/*
  Turns 24 hour time (with minutes) into italian representation
  */
Time2WordItalian(hour, minutes, word)
int hour, minutes;
char *word;
{
  int pm; 
  int rounded = 0;
  char hourWord[MAX_TIMESTRING], minuteWord[MAX_TIMESTRING];
  char bit_of_day[MAX_TIMESTRING];
  *word = '\0';
  
  
  if(!noampm){
	if(hour >= 0  && hour < 5 ) sprintf(bit_of_day," di notte");
	if(hour >= 5  && hour < 12) sprintf(bit_of_day," di mattina");
	if(hour >= 12 && hour < 19) sprintf(bit_of_day," di pomeriggio");
	if(hour >= 19 && hour < 24) sprintf(bit_of_day," di sera");
  }
  
  pm = (hour > 11);
  
  if (pm) {
	hour -= 12;
  }
  if (hour == 0){
	hour = 12;
  }
  
  /* Round to nearest 5 minutes */
  if(approximate){
	if((rounded = minutes%5) >= 3){
	  minutes += (5 - rounded);
	  /* If round to next hour, handle correctly */
	  if(minutes == 60){
		minutes = 0;
		TimeIncrement(&hour, &pm);
	  }
	}
	else if(rounded > 0){
	  minutes -= rounded;
	}
	
	strcat(word, rounded?"circa ":"");
  }
  
  Number2WordItalian(hour, hourWord);
  Number2WordItalian(minutes, minuteWord);
  
  /*
	Try to do some cute things with the time
	*/
  
  if(hour == 1)
	  strcpy(hourWord, "una");
  
  if(minutes == 0){
	if(hour == 12){
	  strcat(word, pm?"mezzogiorno":"mezzanotte");
	  return TRUE;
	}else{         
	  strcat(word, (hour == 1)?"l' ":"le "); 
	  strcat(word, hourWord);
	}
  }
  else if((minutes >= 1) && (minutes <= 9)){
	strcat(word, (hour == 1)?"l' ":"le "); 
	strcat(word, hourWord);
	strcat(word, " e ");
	/*	strcat(word, "zero"); */
	strcat(word, minuteWord);
  }
  else if(minutes == 15){
	strcat(word, (hour == 1)?"l' ":"le "); 
	strcat(word, hourWord);
	strcat(word, " e un quarto");
  }
  else if(minutes == 30){
	strcat(word, (hour == 1)?"l' ":"le "); 
	strcat(word, hourWord);
	strcat(word, " e mezza");
  }
  else if(minutes == 45){
	TimeIncrement(&hour, &pm);
	Number2WordItalian(hour, hourWord);
	if(hour == 12) {
	  strcpy(word, pm?"mezzogiorno":"mezzanotte");
	  strcpy(bit_of_day,"");
	}
	else {
	  if (hour == 1)
		  strcpy(hourWord, "una");
	  strcat(word, (hour == 1)?"l' ":"le "); 
	  strcat(word, hourWord);
	}
	strcat(word, " meno un quarto");
  }
  else if((minutes > 30) && before){
	TimeIncrement(&hour, &pm);
	minutes = 60 - minutes;
	Number2WordItalian(hour, hourWord);
	Number2WordItalian(minutes, minuteWord);
	strcat(word, (hour == 1)?"l' ":"le "); 
	strcat(word, hourWord);
	strcat(word, " meno ");
	strcat(word, minuteWord);
  }
  else{
	strcat(word, (hour == 1)?"l' ":"le "); 
	strcat(word, hourWord);
	strcat(word, " e ");
	strcat(word, minuteWord);
  }
  
  if(!noampm){
	strcat(word,bit_of_day);
  }
  
  return TRUE;
}

/*****************************************************************************
	Klingon Time Routines by Paul E. Black <beehive!paul@cirrus.com>
	Thanks to Captain Krankor <krankor@IMA.ISC.COM>, Larry Oshins
	<Bang@maple.circa.ufl.edu>, and Kevin Iga <mosquito@athena.mit.edu>
	for their input.  Vocabulary and orthography is due to Marc Okrand.

	Since we're talking about terran time, we use terran units (24 hour
	day beginning at midnight), but say it in Klingon.  Klingon is so
	militarized that the use of AM and PM is quite foreign.  Therefore
	time is expressed in a continuous-hour clock (like military time).
 *****************************************************************************/

/*
  Increment the hour (based on 24 hour clock)
*/
static void TimeIncrement24(hPtr)
int *hPtr;
{
  *hPtr = *hPtr%24 + 1;
}

/*
  Converts a number from 0 to 59 into a Klingon string equivalent
*/
static Number2WordKlingon(number, word)
int number;
char *word;
{
  static char *digits[] = {
	"paugh", "wa'", "cha'", "wej", "loS", "vagh", "jav", "Soch",
	"chorgh", "Hut"
	  };
  int tens, units;
  
  *word = '\0';

  if ((number < 0) || (number > 59)){
	return FALSE;
  }
  
  if (number == 0) {
      (void)strcat(word, digits[0]);
      return TRUE;
  }

  tens = number/10;
  units = number%10;
  
  if (tens) {
      (void)sprintf(word, "%smaH", digits[tens]);
  }
  
  if (units) {
      if (tens) {
	  (void)strcat(word, " ");
      }
      (void)strcat(word, digits[units]);
  }
  
  return TRUE;
}

/*
  Turns 24 hour time (with minutes) into Klingonese (Okrand orthography)
*/
Time2WordKlingon(hour, minutes, word)
int hour, minutes;
char *word;
{
    int rounded = 0;
    char hourWord[MAX_TIMESTRING], minuteWord[MAX_TIMESTRING];
  
    *word = '\0';
  
    /* Round to nearest 5 minutes */
    if(approximate){
	rounded = minutes%5;
	if(rounded >= 3){
	  minutes += (5 - rounded);
	  /* If round to next hour, handle correctly */
	  if(minutes == 60){
		minutes = 0;
		TimeIncrement24(&hour);
	  }
	}
	else if(rounded > 0){
	  minutes -= rounded;
	}
	
    }
  
    (void)Number2WordKlingon(hour, hourWord);
    (void)Number2WordKlingon(minutes, minuteWord);
  
    /*
	Try to do some cute things with the time
	*/
    if(minutes == 0){
	  if(hour == 12){
	    (void)strcat(word, "DungluQ");
	  } else if (hour == 0) {
	    (void)strcat(word, "ramjep");
	  } else {
	    (void)strcat(word, hourWord);
	    (void)strcat(word, " rep");
	    if (hour > 1) {
		  (void)strcat(word, "mey");
	    }
	  }
    }
    else if(minutes == 30){
	  (void)strcat(word, hourWord);
	  (void)strcat(word, " je bID");
    }
    else if((minutes > 30) && before){
	  TimeIncrement24(&hour);
	  minutes = 60 - minutes;
	  (void)Number2WordKlingon(hour, hourWord);
	  (void)Number2WordKlingon(minutes, minuteWord);
	  
	  (void)strcat(word, hourWord);
	  (void)strcat(word, " 'ej ratlh "); /* literally: ".. and lacking .." */
	  (void)strcat(word, minuteWord);
    }
    else{
	  (void)strcat(word, hourWord);
	  if (hour == 10 || hour == 20) {
	    (void)strcat(word, " je");	   /* literally: ".. and .." */
	  }
	  (void)strcat(word, " ");
	  (void)strcat(word, minuteWord);
    }
	
    if (rounded) {
	  (void)strcat(word, "Hey");
    }
	
    return TRUE;
  }
